SystemOrganization addCategory: #'LDAP-InternalFilters'!
SystemOrganization addCategory: #'LDAP-BERTypes'!
SystemOrganization addCategory: #LDAP!
SystemOrganization addCategory: #'LDAP-Tests'!

TestCase subclass: #BERTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Tests'!

!BERTest methodsFor: 'as yet unclassified' stamp: 'RHE 2/6/2004 23:07'!
testIntegerEncoding
	|ber stream tests output |
	ber _ BERInteger new.
	stream _ ReadWriteStream on: (String new).

	tests _  self class getIntegerTestSet.
	tests do: [:test | 
		stream resetToStart.
		ber value: (test at: 1).	
		ber writeOn: stream.
		output _  stream contents asString asHex.
		Transcript show: 'Expected: ', (test at: 2), ' Got: ', output; cr.
		self assert: (output = (test at: 2)) ]
	
	! !

!BERTest class methodsFor: 'as yet unclassified' stamp: 'RHE 2/8/2004 20:38'!
getBooleanTestSet
	^  {		{ 0 . '010100' }.
			{ 1 . '0101FF' }.
			{ 255 . '0101FF' }.
			{ 1000 . '0101FF' }.
			{ false . '010100' }.
			{ true . '0101FF' }.
		}.
! !

!BERTest class methodsFor: 'as yet unclassified' stamp: 'RHE 2/6/2004 23:06'!
getIntegerTestSet
	^  {		{27066 .  '020269BA'}. 
			{-27066 .  '02029646'}.
			{72 .  '020148' }.
			{127  . '02017F'}.
			{-128. '020180'}.
			{128  . '02020080'}.
			{ 0 . '020100' }.
			{ 256  .  '02020100'}.
			{4294967290  . '020500FFFFFFFA'}.
			{ 1  . '020101'}.
			{-1 . '0201FF'}.
			{ -129  . '0202FF7F'}.
		}.
! !

!BERTest methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 21:10'!
testSequenceEncoding
	|ber0 ber1 ber2 stream|
	ber0 _ BERSequence new.
	ber1 _ BERInteger new value: 17.
	ber2 _ BERInteger new value: 170.
	
	ber0 addElement: ber1.
	ber0 addElement: ber2.
	stream _ ReadWriteStream on: (String new).

	ber0 writeOn: stream.
	self assert: (stream contents asString asHex = '3007020111020200AA') 

! !

!BERTest class methodsFor: 'as yet unclassified' stamp: 'RHE 1/31/2004 00:17'!
isAbstract
	^ false! !

!BERTest methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 17:16'!
testSearchRequest
	| encoded |
	encoded _ BEREncoder searchRequest: 1 base: 'dc=linalco, dc=test' scope: 2 filter: (LDAPFilter with: 'objectclass' ) attrs: (OrderedCollection new) wantAttrsOnly: false.
	encoded _ encoded asString asHex.

	Transcript show: 'testSearchRequest got: ', encoded; cr.
	self assert: (encoded = '30380201016333041364633D6C696E616C636F2C2064633D746573740A01020A0100020100020100010100870B6F626A656374636C6173733000')
! !

!BERTest methodsFor: 'as yet unclassified' stamp: 'RHE 2/8/2004 19:40'!
testBooleanEncoding
	|ber stream tests output |
	ber _ BERBoolean new.
	stream _ ReadWriteStream on: (String new).

	tests _  self class getBooleanTestSet.
	tests do: [:test | 
		stream resetToStart.
		ber value: (test at: 1).	
		ber writeOn: stream.
		output _  stream contents asString asHex.
		Transcript show: 'Expected: ', (test at: 2), ' Got: ', output; cr.
		self assert: (output = (test at: 2)) ]
	
	! !

!BERTest methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 20:21'!
testBindRequest
	| encoded |
	encoded _ BEREncoder bindRequest: 1 username: 'cn=admin,dc=linalco,dc=test' credentials: 'secret' method: nil.
	encoded _ encoded asString asHex.

	Transcript show: 'testBindRequest got: ', encoded; cr.
	self assert: (encoded = '302D0201016028020103041B636E3D61646D696E2C64633D6C696E616C636F2C64633D746573748006736563726574')
! !

!BERTest class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 20:23'!
testAll
	| b |
	b _ BERTest new.
	
	b 
		testIntegerEncoding;
		testOctetStringEncoding;
		testSequenceEncoding;
		testBooleanEncoding;
		testIntegerDecoding;
		testBindRequest;
		testSearchRequest! !

!BERTest class methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 15:36'!
createLauncher
	| b |
	b _ SimpleButtonMorph new.
	b target: BERTest.
	b actionSelector: #testAll.
	b position: (World bounds bottomRight - (40 @ 40)).
	b label: 'Test'.
	b openInWorld.! !

!BERTest methodsFor: 'as yet unclassified' stamp: 'RHE 2/7/2004 00:31'!
testIntegerDecoding
	|ber stream tests value valueStream byte |
	stream _ ReadWriteStream on: (String new).

	tests _  self class getIntegerTestSet.
	tests do: [:test | 
		valueStream _ ReadStream on: (test at: 2).
		value _ test at: 1.
		stream resetToStart.
		
		[valueStream atEnd] whileFalse: [
			byte _ (valueStream next digitValue * 16).
			byte _ byte + valueStream next digitValue.
			stream nextPut: (byte asCharacter) ].
		
		stream reset.
		ber _ BERDecoder newElementFromEncodedStream: stream.
		self assert: (ber class = BERInteger).
		
		Transcript show: 'Expected: ', (value asString), ' Got: ', (ber value asString); cr.
		self assert: (ber value = value ) 
	]
! !

!BERTest methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 22:48'!
testOctetStringEncoding
	|ber stream tests|
	ber _ BEROctetString new.
	stream _ ReadWriteStream on: (String new).

	tests _ {	{ 'hello' . 5 . '040568656C6C6F' } }.

	tests do: [:test | 
		stream resetToStart.
		ber value: (test at: 1).	
		ber writeOn: stream.
		self assert: (stream contents asString asHex = (test at: 3)) ]
	
	! !

TestCase subclass: #LDAPTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-Tests'!

!LDAPTest class methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 14:44'!
password
	^ 'secret'! !

!LDAPTest methodsFor: 'as yet unclassified' stamp: 'RHE 2/14/2004 14:18'!
testSearch
| conn req |
	conn _ LDAPConnection to: self class hostname port: self class port.
	req _ conn bindAs: self class bindDN credentials: self class password.
	req wait.

	req _ conn newSearch: 'dc=linalco, dc=test' scope: 2 filter: (LDAPFilter with: 'objectclass') attrs: nil  wantAttrsOnly: false.
	req wait.
	Transcript show: req result; cr.
	req result inspect.
	
	req _ conn newSearch: 'dc=linalco, dc=test' scope: 2 filter: (LDAPFilter with: 'objectclass') attrs: nil  wantAttrsOnly: false.
	[req hasMoreEntries] whileTrue: [
		| r |
		 r _ conn next.
		r inspect
		Transcript show: '---------------------------', r dn; cr ].
	
	conn disconnect
	! !

!LDAPTest class methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 14:44'!
bindDN
	^ 'cn=admin,dc=linalco,dc=test'! !

!LDAPTest class methodsFor: 'as yet unclassified' stamp: 'RHE 2/13/2004 16:23'!
port
	^ 12000! !

!LDAPTest class methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 14:42'!
hostname
	^ '127.0.0.1'
	! !

Error subclass: #LDAPException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

Object subclass: #LDAPRequest
	instanceVariableNames: 'worker messageId waitingSemaphore isDone result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

!LDAPRequest methodsFor: 'as yet unclassified' stamp: 'RHE 2/13/2004 18:38'!
wantsMoreResponses
	"you most likely not want to implement this in a subclass, but processResponse:"
	^ self isDone not! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: 'RHE 2/8/2004 18:48'!
result
	self wait.
	^ result! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: 'RHE 2/13/2004 18:38'!
hasResult
	^ isDone! !

LDAPRequest subclass: #LDAPUnbindRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

!LDAPUnbindRequest methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 14:19'!
initializeWith: anLDAPWorker 
	| encoded |
	worker _ anLDAPWorker.
	messageId _ worker nextMessageId.
	encoded _ BEREncoder unbindRequest: messageId.
	Transcript show: encoded asHex; cr.
	worker send: encoded.
! !

LDAPRequest subclass: #LDAPSearchRequest
	instanceVariableNames: 'partialResults endMarker iterator iteratorSemaphore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

!LDAPSearchRequest methodsFor: 'as yet unclassified' stamp: 'RHE 2/14/2004 14:15'!
forEachDo: aBlock
	self notYetImplemented! !

!LDAPSearchRequest methodsFor: 'as yet unclassified' stamp: 'RHE 2/14/2004 02:26'!
next
	"if squeak ever gets really multithreaded, we have to fix this"
	
	"this stinks"
	
	 (( (partialResults isNil) or: [partialResults size = iterator])  and: [isDone] )  ifTrue: [ ^ nil ].
	partialResults size = iterator ifTrue: [
	iteratorSemaphore wait.
	 (( (partialResults isNil) or: [partialResults size = iterator])  and: [isDone] )  ifTrue: [ ^ nil ].
	].
	iterator _ iterator + 1.
	^ (partialResults at: iterator)
! !

!LDAPSearchRequest methodsFor: 'as yet unclassified' stamp: 'RHE 2/13/2004 18:55'!
hasMoreEntries
	"if squeak ever gets really multithreaded, we have to fix this"
	| |
	 (( (partialResults isNil) or: [partialResults size = iterator])  and: [isDone] )  ifTrue: [ ^ false ].

	partialResults size = iterator ifTrue: [
	iteratorSemaphore wait.
	 (( (partialResults isNil) or: [partialResults size = iterator])  and: [isDone] )  ifTrue: [ ^ false ].
	].

	^ true
	! !

!LDAPSearchRequest methodsFor: 'as yet unclassified' stamp: 'RHE 2/13/2004 18:40'!
processResponse: aSearchResultEntry
	aSearchResultEntry class isEndMarker
		ifTrue: [ isDone _ true ]
		ifFalse: [ partialResults addLast: aSearchResultEntry ].
		
	iteratorSemaphore signal! !

!LDAPSearchRequest methodsFor: 'as yet unclassified' stamp: 'RHE 2/13/2004 17:44'!
initializeWith: anLDAPWorker base: aBase scope: aSearchScope filter: aFilter attrs: someAttrs wantAttrsOnly: wantsAttrsOnly
	| encoded |
	partialResults _ OrderedCollection new.
	iterator _  0.
	iteratorSemaphore _ Semaphore new. "forMutualExclusion."
	self initializeWith: anLDAPWorker.
	encoded _ BEREncoder searchRequest: messageId  base: aBase scope: aSearchScope filter: aFilter attrs: someAttrs wantAttrsOnly: wantsAttrsOnly.
	Transcript show: encoded asHex; cr.
	anLDAPWorker send: encoded.
! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 16:48'!
processResponse: aMessage
	result _ aMessage! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: 'RHE 2/13/2004 18:45'!
isDone
	^ isDone! !

LDAPRequest subclass: #LDAPBindRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

!LDAPBindRequest methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 20:46'!
initializeWith: anLDAPWorker username: aUsername credentials: aPassword method: anAuthMethod
	anAuthMethod ifNotNil: [ self notYetImplemented ].
	self initializeWith: anLDAPWorker.
	anLDAPWorker send:
		(BEREncoder bindRequest: messageId username: aUsername credentials: aPassword method: anAuthMethod).
! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: 'RHE 2/8/2004 18:48'!
wait
	result ifNil: [
		waitingSemaphore wait ]! !

!LDAPRequest methodsFor: 'as yet unclassified' stamp: 'RHE 2/13/2004 18:37'!
initializeWith: anLDAPWorker
	worker _ anLDAPWorker.
	messageId _ worker nextMessageId.
	waitingSemaphore _ Semaphore new.
	isDone _ false.

	worker addTarget: self withLock: waitingSemaphore for: messageId.

	! !

Object subclass: #LDAPFilter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 16:41'!
with: anAttrType lessOrEqualThan: anAttrValue
	^ LDAPFilterLessOrEqual new setType: anAttrType value: anAttrValue! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 16:29'!
containing: aSubstring atAnywhereOf: anAttrType
	^ LDAPFilterSubstrings containing: aSubstring atAnywhereOf: anAttrType! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 16:30'!
containing: aSubstring atStartOf: anAttrType
	^ LDAPFilterSubstrings containing: aSubstring atStartOf: anAttrType! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 16:40'!
with: anAttrType equalTo: anAttrValue
	^ LDAPFilterEquality new setType: anAttrType value: anAttrValue! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 16:40'!
with: anAttrType greaterOrEqualThan: anAttrValue
	^ LDAPFilterGreaterOrEqual new setType: anAttrType value: anAttrValue
	! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 17:15'!
orOf: someFilters
	| set |
	set _ LDAPFilterOr new.
	someFilters do: [ :each | set add: each ].
	^ set
! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 11:26'!
not: aFilter
	^ LDAPFilterNot filter: aFilter! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 16:30'!
containing: aSubstring atEndOf: anAttrType
	^ LDAPFilterSubstrings containing: aSubstring atEndOf: anAttrType! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 17:14'!
andOf: someFilters
	| set |
	set _ LDAPFilterAnd new.
	someFilters do: [ :each | set add: each ].
	^ set
! !

!LDAPFilter class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 16:29'!
with: anAttrType
	^ LDAPFilterPresent having: anAttrType! !

Object subclass: #LDAPFilterSingle
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

LDAPFilterSingle subclass: #LDAPFilterWithSubstring
	instanceVariableNames: 'type choiceTag substring'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

!LDAPFilterWithSubstring methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 17:17'!
asBERElement
	| ber substrSeq |
	ber _ BERSequence new.
	ber addElement: (BEROctetString new value: type).
	
	substrSeq _ BERSequence new.
	substrSeq addElement: (BEROctetString new value: substring) tagSetContext withTag: choiceTag.
	
	ber addElement: substrSeq.
	^ ber! !

LDAPFilterWithSubstring subclass: #LDAPFilterSubstrings
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

!LDAPFilterSubstrings class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 16:03'!
containing: aSubstring atStartOf: anAttrType
	^ self new setSubstring: aSubstring type: anAttrType choiceTag: 0
! !

!LDAPFilterSubstrings class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 16:03'!
containing: aSubstring atEndOf: anAttrType
	^ self new setSubstring: aSubstring type: anAttrType choiceTag: 2
! !

!LDAPFilterSubstrings class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 16:03'!
containing: aSubstring atAnywhereOf: anAttrType
	^ self new setSubstring: aSubstring type: anAttrType choiceTag: 1
! !

!LDAPFilterSubstrings class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 19:44'!
tagValue
	^ 4! !

!LDAPFilterWithSubstring methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 15:51'!
setSubstring: aSubstring type: anAttrType choiceTag: aChoiceTag
	substring _ aSubstring.
	type _ anAttrType.
	choiceTag _ aChoiceTag
! !

LDAPFilterSingle subclass: #LDAPFilterWithFilter
	instanceVariableNames: 'filter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

LDAPFilterWithFilter subclass: #LDAPFilterNot
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

!LDAPFilterNot class methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 11:25'!
tagValue
	^ 2! !

!LDAPFilterNot class methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 11:28'!
filter: aFilter
	^ self new setFilter: aFilter! !

!LDAPFilterWithFilter methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 11:24'!
asBERElement
	| ber |
	ber _ BERSet new.
	ber addElement: (filter asBERElement) tagSetContext withTag: filter class tagValue.
	
	^ ber! !

!LDAPFilterWithFilter methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 11:29'!
setFilter: aFilter
	filter _ aFilter! !

LDAPFilterSingle subclass: #LDAPFilterWithSet
	instanceVariableNames: 'filters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

!LDAPFilterWithSet methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 19:46'!
initialize
	filters _ OrderedCollection new
	! !

LDAPFilterWithSet subclass: #LDAPFilterOr
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

!LDAPFilterOr class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 19:44'!
tagValue
	^ 1! !

LDAPFilterWithSet subclass: #LDAPFilterAnd
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

!LDAPFilterAnd class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 19:41'!
tagValue
	^ 0! !

!LDAPFilterWithSet methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 19:46'!
add: aFilter
	filters addLast: aFilter! !

!LDAPFilterWithSet class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 19:49'!
new
	^ self basicNew initialize! !

!LDAPFilterWithSet methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 19:22'!
asBERElement
	| ber |
	ber _ BERSet new.
	filters do: [ :each | ber addElement: (each asBERElement) tagSetContext withTag: each class tagValue ].
	^ ber! !

LDAPFilterSingle subclass: #LDAPFilterWithType
	instanceVariableNames: 'type'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

LDAPFilterWithType subclass: #LDAPFilterPresent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

!LDAPFilterPresent class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 16:04'!
having: anAttrType
	^ self new setType: anAttrType
! !

!LDAPFilterPresent class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 19:44'!
tagValue
	^ 7! !

!LDAPFilterWithType methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 15:51'!
setType: anAttrType
	type _ anAttrType
	! !

!LDAPFilterWithType methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 20:34'!
asBERElement
	| ber |
	ber _ BEROctetString new value: type.
	^ ber! !

!LDAPFilterSingle class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 15:00'!
tagValue
	self subclassResponsibility! !

!LDAPFilterSingle methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 20:56'!
asBERElement
	self subclassResponsibility! !

LDAPFilterSingle subclass: #LDAPFilterWithValueAssertion
	instanceVariableNames: 'type value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

LDAPFilterWithValueAssertion subclass: #LDAPFilterApprox
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

!LDAPFilterApprox class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 19:42'!
tagValue
	^ 8! !

LDAPFilterWithValueAssertion subclass: #LDAPFilterLessOrEqual
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

!LDAPFilterLessOrEqual class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 19:43'!
tagValue
	^ 6! !

LDAPFilterWithValueAssertion subclass: #LDAPFilterEquality
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

!LDAPFilterEquality class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 19:43'!
tagValue
	^ 3! !

!LDAPFilterWithValueAssertion methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 19:19'!
asBERElement
	| ber |
	ber _ BERSequence new.
	ber
		addElement: (BEROctetString new value: type);
		addElement: (BEROctetString new value: value).
	^ ber! !

!LDAPFilterWithValueAssertion methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 16:04'!
setType: anAttrType value: anAttrValue
	type _ anAttrType.
	value _ anAttrValue	! !

LDAPFilterWithValueAssertion subclass: #LDAPFilterGreaterOrEqual
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-InternalFilters'!

!LDAPFilterGreaterOrEqual class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 19:43'!
tagValue
	^ 5
	! !

Object subclass: #BERDecoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

!BERDecoder class methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 14:07'!
identifyIncomingElement: firstByte
	| classes type |
	
	classes _ {
		BERInteger. BEROctetString . BERSequence. BEREnumerated. BERBoolean. BERSet. BERNull.
		LDAPBindResult. LDAPSearchResultEntry. LDAPSearchResultCode.
	}.
	
	type _ classes detect: [ :each | each tagValue = firstByte asInteger ] ifNone: [nil].
	^ type
	! !

!BERDecoder class methodsFor: 'as yet unclassified' stamp: 'RHE 2/7/2004 01:46'!
newElementFromEncodedStream: aStream
	| firstByte element elementClass |
	firstByte _ aStream next.
	Transcript show: 'First byte read: ', firstByte  hex; cr.
	elementClass _ BERDecoder identifyIncomingElement: firstByte.
	elementClass ifNil: [
		LDAPException new errorMessage: 'invalid tag'; signal ].
	element _ elementClass newFromEncodedStream: aStream.
	^ element
! !

Object subclass: #BERElement
	instanceVariableNames: 'length lengthLength value tagHeader'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

BERElement subclass: #BERNull
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BERTypes'!

!BERNull class methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 14:04'!
tagValue
	^ 5! !

!BERNull class methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 14:08'!
newFromEncodedStream: aStream
	| obj |

	Transcript show: 'Reading BERNull'; cr.
	
	obj _ self new.
	obj readLengthFrom: aStream.
"not sure about this.. should it be 0?"
"	obj length: 1."
	obj value: nil.
	
	^ obj
! !

!BERNull methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 14:08'!
writeBodyOn: aStream
	self writeLength: 0 on: aStream! !

BERElement subclass: #BEROctetString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BERTypes'!

!BEROctetString class methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 22:48'!
tagValue
	^ 4 ! !

!BEROctetString methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 20:23'!
writeBodyOn: aStream
	self writeLength: (value size) on: aStream.
	aStream nextPutAll: value.
	"theorically we should convert it to UTF8"
	^ value size! !

!BEROctetString class methodsFor: 'as yet unclassified' stamp: 'RHE 2/7/2004 01:55'!
newFromEncodedStream: aStream
	| obj elementLen |
	
	Transcript show: 'Reading BEROctetString'; cr.
	
	obj _ self new.
	elementLen _ obj readLengthFrom: aStream.
	obj value: (aStream next: elementLen).
	Transcript show: 'read string: ', obj value; cr.
	^ obj! !

!BERElement methodsFor: 'as yet unclassified' stamp: 'RHE 2/7/2004 02:55'!
lengthLength
	^ lengthLength! !

BERElement subclass: #BERInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BERTypes'!

!BERInteger class methodsFor: 'as yet unclassified' stamp: 'RHE 2/7/2004 02:01'!
newFromEncodedStream: aStream
	| obj value elementLen highBitPos xorMask |

	Transcript show: 'Reading BERInteger'; cr.
	
	obj _ self new.
	elementLen _ obj readLengthFrom: aStream.

	value _ 0.
	(1 to: elementLen) do: [ :i | value _ (value bitShift: 8) + aStream next asInteger ].

	"if the high bit is set, we have negative"	
	highBitPos _ value highBit.
	(highBitPos  = (elementLen * 8))  ifTrue: [
		xorMask _ (1 bitShift: highBitPos) - 1.
		value _ value bitXor: xorMask.
		value _ (value + 1) negated.
	].
	
	obj 
		value: value.
	^ obj
! !

!BERInteger class methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 20:30'!
tagValue
	^ 2 ! !

!BERInteger methodsFor: 'as yet unclassified' stamp: 'RHE 2/6/2004 22:39'!
writeBodyOn: aStream
	| octets netOctets isNegative remainderValue octetsIndex |
	isNegative _ value < 0.
	octets _ ByteArray new: (value digitLength + 2).
	
	"put value into octet array, covert negatives as appropiate"
	
	isNegative not 
		ifTrue: [
			value = 0
				ifTrue: [
					octetsIndex _ 1.
					octets at: octetsIndex put: 0 ]
				ifFalse: [
					remainderValue _ value.
					octetsIndex _ 0.
					[remainderValue >  0] whileTrue: [
						octetsIndex _ octetsIndex + 1.
						octets at: octetsIndex put: (remainderValue bitAnd: 255).
						remainderValue _ remainderValue bitShift: -8 ] ]
					]
		ifFalse: [ 
			"negatives are in two's complement -- to convert: 1. change to positive. 2. substract 1, 3. xor everythnig"
			remainderValue _ value negated.
			remainderValue _ remainderValue - 1.
			octetsIndex _ 0.
			[
				octetsIndex _ octetsIndex + 1.
				octets at: octetsIndex put: ((remainderValue bitXor: 255) bitAnd: 255).
				remainderValue _ remainderValue bitShift: -8.
				remainderValue > 0.
			] whileTrue ].
	
	"if originally we had a positive, and highest bit is set in the beginning of the array, we prefix the array with a zero byte"
	"if said bit is set and original was negative, prefix with a all-ones byte"
	"we actually test the end of the array because we are switching it around later for network order"

	value > 0 ifTrue: [
		(((octets at: octetsIndex) bitAnd: 128) > 0) ifTrue: [
			octetsIndex _ octetsIndex + 1.
			octets at: octetsIndex put: 0 ] ].
	value < 0 ifTrue: [
		(((octets at: octetsIndex) bitAnd: 128) = 0) ifTrue: [
			octetsIndex _ octetsIndex + 1.
			octets at: octetsIndex put: 255 ] ].		

	"hton"
	netOctets _ ByteArray new: octetsIndex.
	(1 to: octetsIndex) 
		do: [:i | netOctets at: i put: (octets at: (octetsIndex + 1) - i) ].
	
	self writeLength: octetsIndex on: aStream.
	aStream nextPutAll: (netOctets asString).
	^ octetsIndex! !

BERInteger subclass: #BEREnumerated
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BERTypes'!

!BEREnumerated class methodsFor: 'as yet unclassified' stamp: 'RHE 2/7/2004 02:29'!
newFromEncodedStream: aStream
	| obj |
	Transcript show: 'Reading BEREnumerated'; cr.
	obj  _  super newFromEncodedStream: aStream.
	^ obj! !

!BEREnumerated class methodsFor: 'as yet unclassified' stamp: 'RHE 2/7/2004 01:28'!
tagValue
	^ 10! !

!BERElement methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 19:42'!
tagSetContext
	tagHeader _ tagHeader bitOr: 16r80
		! !

!BERElement class methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 20:51'!
new
	^ self basicNew initialize! !

BERElement subclass: #LDAPMessage
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

LDAPMessage subclass: #LDAPSearchResultEntry
	instanceVariableNames: 'dn attrs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

!LDAPSearchResultEntry class methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 11:55'!
isEndMarker
	^ false! !

!LDAPSearchResultEntry class methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 13:37'!
newFromEncodedStream: aStream
	| obj berDN berAttrs attrs |
	obj _ self new.
	obj readLengthFrom: aStream.
	
	berDN _ BERDecoder newElementFromEncodedStream: aStream.
	berAttrs _ BERDecoder newElementFromEncodedStream: aStream.

	"heriarchy hell"
	
	attrs _ OrderedCollection new.
	berAttrs elements do: [ :eachBerAttr | | values |
		
		values _ OrderedCollection new.
		((eachBerAttr value) elements at: 2) value elements do: [ :eachBerValue |	
			values add: (eachBerValue value value).
		].
		
		attrs add: (Association new;
			key: 	(((eachBerAttr value) elements at: 1) value value)
			value: values)	
	].
	
	obj setDN: berDN value attrs: attrs.
	
	^ obj! !

!LDAPSearchResultEntry methodsFor: 'as yet unclassified' stamp: 'RHE 2/8/2004 21:10'!
dn
	^ dn! !

!LDAPSearchResultEntry class methodsFor: 'as yet unclassified' stamp: 'RHE 2/8/2004 20:39'!
tagValue
	^ 16r64! !

!LDAPSearchResultEntry methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 12:00'!
setDN: aDN attrs: someAttrs
	dn _ aDN.
	attrs _ someAttrs! !

!LDAPSearchResultEntry methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 11:59'!
attrs
	^ attrs! !

!LDAPMessage methodsFor: 'as yet unclassified' stamp: 'RHE 2/7/2004 01:09'!
id: aMessageId
	id _ aMessageId! !

!LDAPMessage methodsFor: 'as yet unclassified' stamp: 'RHE 2/7/2004 01:09'!
id
	^ id! !

!LDAPMessage class methodsFor: 'as yet unclassified' stamp: 'RHE 2/7/2004 03:05'!
newFrom: aStream
	| ber mesg |
	ber _ BERDecoder newElementFromEncodedStream: aStream.
	mesg _( (ber elements at: 2) value).
	mesg id: ((ber elements at: 1) value value).
	^ mesg! !

LDAPMessage subclass: #LDAPResult
	instanceVariableNames: 'resultCode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

!LDAPResult methodsFor: 'as yet unclassified' stamp: 'RHE 2/8/2004 20:34'!
resultCode
	^ resultCode! !

!LDAPResult methodsFor: 'as yet unclassified' stamp: 'RHE 2/8/2004 20:34'!
resultCode: aCode
	resultCode _ aCode! !

LDAPResult subclass: #LDAPBindResult
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

!LDAPBindResult class methodsFor: 'as yet unclassified' stamp: 'RHE 2/7/2004 02:05'!
tagValue
	^ 16r61! !

!LDAPBindResult class methodsFor: 'as yet unclassified' stamp: 'RHE 2/7/2004 02:34'!
newFromEncodedStream: aStream
	| obj ber |
	obj _ self new.
	obj readLengthFrom: aStream.
	ber _ BERDecoder newElementFromEncodedStream: aStream.
	obj resultCode: (ber value).
	BERDecoder newElementFromEncodedStream: aStream.
	BERDecoder newElementFromEncodedStream: aStream.
	
	"resultCode -- matchedDN -- errormessage"	
	^ obj! !

LDAPResult subclass: #LDAPSearchResultCode
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

!LDAPSearchResultCode class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 18:17'!
tagValue
	^ 16r65! !

!LDAPSearchResultCode class methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 11:55'!
isEndMarker
	^ true! !

!LDAPSearchResultCode class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 18:38'!
newFromEncodedStream: aStream
	| obj ber |
	obj _ self new.
	obj readLengthFrom: aStream.
	ber _ BERDecoder newElementFromEncodedStream: aStream.
	obj resultCode: (ber value).
	
	BERDecoder newElementFromEncodedStream: aStream.
	BERDecoder newElementFromEncodedStream: aStream.
	
	"resultCode -- matchedDN -- errormessage"	
	"this is the same as in LDAPBindResult ... ugh"
	
	^ obj! !

!BERElement methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 14:57'!
writeBodyOn: aStream
	self subclassResponsibility! !

!BERElement methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 20:49'!
initialize
	tagHeader _ 0
! !

!BERElement methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 21:19'!
writeOn: aStream
	aStream nextPut: ((self class tagValue bitOr: tagHeader) asCharacter).
	self writeBodyOn: aStream
! !

!BERElement methodsFor: 'as yet unclassified' stamp: 'RHE 1/30/2004 23:41'!
value: aValue
	value _ aValue! !

!BERElement methodsFor: 'as yet unclassified' stamp: 'RHE 2/7/2004 02:55'!
readLengthFrom: aStream
	| octets  firstOctet |
	
	firstOctet _ aStream next asInteger.
	firstOctet < 128
		
		"short definite length"
		ifTrue: [ length _ firstOctet. lengthLength _ 1. ]
		
		"long definite length"
		ifFalse: [
			octets _ aStream next: (firstOctet bitXor: 128).
			lengthLength _ (firstOctet bitXor: 128).
			length _ octets contents inject: 0 into: [:injectedValue :each | (injectedValue bitShift: 8) + each]
		].
		
	^ length			! !

!BERElement methodsFor: 'as yet unclassified' stamp: 'RHE 2/6/2004 18:52'!
writeLength: aLength on: aStream
	| octets octetsIndex remainderValue netOctets |
	
	octetsIndex _ 1.
	aLength < 128
		
		"short definite length"
		ifTrue: [ aStream nextPut: aLength asCharacter ]
		
		"long definite length"
		ifFalse: [
			octets _ ByteArray new: (value digitLength + 1).
			remainderValue _ value.
			[remainderValue > 0] whileTrue: [
				octets at: octetsIndex  put: (remainderValue bitAnd: 16rFF).
				octetsIndex _ octetsIndex + 1.
				remainderValue _ remainderValue bitShift: -8 ].
			octets at: octetsIndex put: (octetsIndex - 1).
			
			"hton"
			netOctets _ ByteArray new: octetsIndex.
			(1 to: octetsIndex)
				do: [:i | netOctets at: i put: (octets at: (octetsIndex + 1) - i) ].
			
			aStream nextPutAll: (netOctets asString).
		].
		
	^ octetsIndex			! !

!BERElement methodsFor: 'as yet unclassified' stamp: 'RHE 1/30/2004 16:08'!
length
	^ length! !

!BERElement methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 19:19'!
tagSetApplication
	tagHeader _ tagHeader bitOr: 16r40! !

!BERElement methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 23:03'!
writeOn: aStream withTag: aTag
	| combinedTag |
	combinedTag _ self class tagValue bitOr: tagHeader.

"	here we are supposing that if we are given a tag, we dont need the universal tag value
	 im not really sure on wether its correct or not"			
	
	aTag ifNotNil: [
		combinedTag _ ((combinedTag bitOr: 31)  bitXor: 31).
		combinedTag _ combinedTag bitOr: aTag ].

	aStream nextPut: (combinedTag asCharacter).
	self writeBodyOn: aStream
! !

BERElement subclass: #BERBoolean
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BERTypes'!

!BERBoolean class methodsFor: 'as yet unclassified' stamp: 'RHE 2/8/2004 19:28'!
tagValue
	^ 1! !

!BERBoolean methodsFor: 'as yet unclassified' stamp: 'RHE 2/8/2004 20:37'!
writeBodyOn: aStream
	self writeLength: 1 on: aStream.
	(value = 0 or: [value = false])
		ifTrue: [aStream nextPut: (0 asCharacter)]
		ifFalse: [aStream nextPut: (255 asCharacter)].
! !

!BERBoolean class methodsFor: 'as yet unclassified' stamp: 'RHE 2/8/2004 19:36'!
newFromEncodedStream: aStream
	| obj value |

	Transcript show: 'Reading BERBoolean'; cr.
	
	obj _ self new.
	obj readLengthFrom: aStream.
	value _ aStream next.

	obj 
		value: value > 0.
	^ obj
! !

!BERElement class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 14:58'!
newFromEncodedStream: aStream
	self subclassResponsibility! !

!BERElement methodsFor: 'as yet unclassified' stamp: 'RHE 1/30/2004 16:08'!
value
	^ value! !

!BERElement class methodsFor: 'as yet unclassified' stamp: 'RHE 2/11/2004 14:58'!
tagValue
	self subclassResponsibility! !

BERElement subclass: #BERConstruct
	instanceVariableNames: 'elements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BERTypes'!

!BERConstruct class methodsFor: 'as yet unclassified' stamp: 'RHE 1/31/2004 17:45'!
new
	^ self basicNew initialize! !

BERConstruct subclass: #BERSet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BERTypes'!

!BERSet class methodsFor: 'as yet unclassified' stamp: 'RHE 2/8/2004 22:36'!
tagValue
	"SET + Constructed"
	^ (17 + 32)! !

!BERSet class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 18:08'!
newFromEncodedStream: aStream
	Transcript show: 'Reading BERSet'; cr.
	^ super newFromEncodedStream: aStream! !

BERConstruct subclass: #BERSequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP-BERTypes'!

!BERSequence class methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 21:17'!
tagValue
	"SEQUENCE + Constructed"
	^ (16 + 32)! !

!BERSequence class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 18:07'!
newFromEncodedStream: aStream
	Transcript show: 'Reading BERSequence'; cr.
	^ super newFromEncodedStream: aStream! !

!BERConstruct methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 22:43'!
addElement: anElement
	self addElement: anElement withTag: nil! !

!BERConstruct methodsFor: 'as yet unclassified' stamp: 'RHE 2/7/2004 03:03'!
elements
	^ elements! !

!BERConstruct methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 20:09'!
addElement: anElement withTag: aContextTag
	| taggedElement |
	taggedElement _ Association new;
		key: aContextTag value: anElement.
	elements addLast: taggedElement! !

!BERConstruct methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 20:49'!
initialize
	super initialize.
	elements _ OrderedCollection new! !

!BERConstruct methodsFor: 'as yet unclassified' stamp: 'RHE 2/1/2004 21:18'!
writeBodyOn: aStream
	| data dataStream |
	data _ Array new.
	dataStream _ WriteStream on: data.
	
	elements do: [:taggedElement |
		taggedElement value writeOn: dataStream withTag: (taggedElement key) ].
	
	"shouldnt we move this somewhere else?"
	self writeLength: (dataStream contents size) on: aStream.
	aStream nextPutAll: (dataStream contents)! !

!BERConstruct class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 18:06'!
newFromEncodedStream: aStream
	| obj elementLen part |
	
	Transcript show: 'Reading BERConstruct'; cr.
		
	obj _ self new.
	obj readLengthFrom: aStream.
	
	elementLen _ obj length.
	[elementLen > 0]
		whileTrue: [
			part _ BERDecoder newElementFromEncodedStream: aStream.
			elementLen _ elementLen - part length.
			elementLen _ elementLen - 1 - part lengthLength. "tag"
			obj addElement: part.
		].
	^ obj
	! !

Object subclass: #LDAPWorker
	instanceVariableNames: 'sockStream hostname port nextMessageId lock targets running'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

!LDAPWorker methodsFor: 'as yet unclassified' stamp: 'RHE 2/8/2004 18:37'!
nextMessageId
	"here we should check wether we're past the message id limit, and if so reconnect and bind"
	nextMessageId _ nextMessageId + 1.
	^ nextMessageId! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: 'RHE 2/5/2004 23:28'!
stop
	running _ false.! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 13:54'!
disconnect
	self stop
	! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: 'RHE 2/5/2004 23:28'!
running
	^ running! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: 'RHE 2/5/2004 22:58'!
send: someData
	lock critical: [
		sockStream 
			nextPutAll: someData;
			flush.
	]! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 13:47'!
connect
	[ sockStream _ SocketStream openConnectionToHostNamed: hostname port: port ]
		on: Exception do: [:ex | sockStream _ nil. self error: ex messageText].
	sockStream 
		binary;
		autoFlush: false;
		noTimeout
! !

!LDAPWorker class methodsFor: 'as yet unclassified' stamp: 'RHE 2/5/2004 13:58'!
newTo: aHostname port: aPort
	^ self basicNew initialize: aHostname port: aPort! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 18:36'!
start
	| process TTL |
	running _ true.
	TTL _ 4. "this is just a develoment hack so i dont have to kill it each time"
	process _ [ 
		[running] whileTrue: [
			|  element  |
			element _ LDAPMessage newFrom: sockStream.
			Transcript show: '********* INCOMING: ', element asString; cr. 
			self dispatchMessage: element.
			TTL < 1 ifTrue: [
			self running: false.] ifFalse: [ TTL _ TTL -1].
		]
	] fork.
	^ process
	! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: 'RHE 2/5/2004 23:28'!
initialize: aHostname port: aPort
	hostname _ aHostname.
	port _ aPort.
	nextMessageId _ 0.
	lock _ Semaphore forMutualExclusion.
	targets _ Dictionary new.
	running _ false.! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: 'RHE 2/5/2004 23:20'!
addTarget: aRequest withLock: aSemaphore for: aMessageId
	lock critical: [
		| data |
		targets at: aMessageId ifPresent: [
			Error new;
				messageText: 'messageId already present -- shouldnt happen'; 
				signal.
		].
		data _ Dictionary new.
		data
			at: #request put: aRequest;
			at: #semaphore put: aSemaphore.
		targets at: aMessageId put: data.
	]! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 14:24'!
dispatchMessage: aMessage
	| target id who |
	id _ aMessage id.
	target _targets at: id
		ifAbsent: [self error: 'unexpected message'].
	
	"why do we want to signal here, instead in processResponse?"
	"weakify targets?"
	who _ target at: #request.
	who processResponse: aMessage.
	(target at: #semaphore) signal.
	
	who wantsMoreResponses
		ifFalse: [targets removeKey: id].

! !

!LDAPWorker methodsFor: 'as yet unclassified' stamp: 'RHE 2/5/2004 23:28'!
running: aBoolean
	running _ aBoolean! !

Object subclass: #RecursionLock
	instanceVariableNames: 'lockedByProcess semaphore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

!RecursionLock commentStamp: 'RHE 2/3/2004 13:09' prior: 0!
I work as Semaphore forMutualExclusion but I won't block in critical: if I was already locked by the same process running me now.	

	|lock|

	lock _ RecursionLock new.
	lock critical:[
		Transcript show: 'Locked.';cr.
		lock critical:[
			Transcript show: 'By now I would be  deadlocked if I was a Semaphore.';cr. ] ]!

!RecursionLock class methodsFor: 'instance creation' stamp: 'RHE 2/3/2004 12:02'!
new
	^ self basicNew initialize! !

!RecursionLock methodsFor: 'initialize-release' stamp: 'RHE 2/3/2004 11:39'!
initialize
	semaphore _ Semaphore forMutualExclusion! !

!RecursionLock methodsFor: 'mutual-exclusion' stamp: 'RHE 2/3/2004 13:04'!
critical: protectedBlock
	| blockValue |
	lockedByProcess = Processor activeProcess
		ifTrue: [
			"If lockedByProcess matches, its obvously not nil and thereforey we were already locked.
			So skip the semaphore."
			blockValue _ protectedBlock value ]
		ifFalse: [ 
			"Otherwise lock and proceed as usual making sure we reset the lock once its over"
			semaphore critical: [
				lockedByProcess _ Processor activeProcess.
				blockValue _  [protectedBlock value] ensure: [lockedByProcess _ nil] ] ].
	^ blockValue
! !

Object subclass: #BEREncoder
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

!BEREncoder class methodsFor: 'as yet unclassified' stamp: 'RHE 2/10/2004 20:44'!
searchRequest: aMessageId base: aBase scope: aSearchScope filter: aFilter attrs: someAttrs wantAttrsOnly: wantsAttrsOnly
	| stream mesg req attrsSeq |
	stream _ ReadWriteStream on: (String new).

	mesg _ BERSequence new.
	mesg addElement: (BERInteger new value: aMessageId).

	req _ BERSequence new tagSetApplication.
	
	req
		addElement: (BEROctetString new value: aBase);
		addElement: (BEREnumerated new value: aSearchScope);
		addElement: (BEREnumerated new value: 0); "derefAliases"
		addElement: (BERInteger new value: 0); "sizelimit"
		addElement: (BERInteger new value: 0); "timelimit"
		addElement: (BERBoolean new value: false); "typesonly boolean"
		addElement: 
			(aFilter asBERElement) tagSetContext withTag: (aFilter class tagValue).

	attrsSeq _ BERSequence new.
	someAttrs do: [ :each |
		attrsSeq addElement: (BEROctetString new value: each) ].
	req addElement: attrsSeq.

	mesg addElement: req withTag: 3.
	mesg writeOn: stream.
	
	^ stream contents
	! !

!BEREncoder class methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 14:18'!
unbindRequest: aMessageId
	| stream mesg req |
	stream _ ReadWriteStream on: (String new).

	mesg _ BERSequence new.
	mesg addElement: (BERInteger new value: aMessageId).

	req _ BERSequence new tagSetApplication.
	req addElement: (BERNull new).

	mesg addElement: req withTag: 2.
	mesg writeOn: stream.
		
	^ stream contents! !

!BEREncoder class methodsFor: 'as yet unclassified' stamp: 'RHE 2/8/2004 19:25'!
bindRequest: aMessageId username: aUsername credentials: aPassword method: anAuthMethod
	| stream mesg req |
	stream _ ReadWriteStream on: (String new).

	mesg _ BERSequence new.
	mesg addElement: (BERInteger new value: aMessageId).

	req _ BERSequence new tagSetApplication.
	req addElement: (BERInteger new value: 3).
	req addElement: (BEROctetString new value: aUsername).
	req addElement: (BEROctetString new tagSetContext;  value: aPassword) withTag: 0.

	mesg addElement: req withTag: 0.
	mesg writeOn: stream.
		
	^ stream contents! !

Object subclass: #LDAPConnection
	instanceVariableNames: 'worker workerThread'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'LDAP'!

!LDAPConnection methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 14:49'!
newSearch: aBase scope: aSearchScope filter: aFilter attrs: someAttrs wantAttrsOnly: wantsAttrsOnly
	| nonNilFilter nonNilAttrs |
	nonNilFilter _ aFilter
		ifNil: [ nonNilFilter _ LDAPFilter with: 'objectclass' ]
		ifNotNil: [ aFilter ].
	
	nonNilAttrs _ someAttrs
		ifNil: [ nonNilAttrs _ OrderedCollection new ]
		ifNotNil: [ someAttrs ].
	
	^ LDAPSearchRequest new initializeWith: worker base: aBase scope: aSearchScope filter: nonNilFilter attrs: nonNilAttrs wantAttrsOnly: wantsAttrsOnly! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: 'RHE 2/6/2004 16:15'!
initializeWithHost: aHostname port: aPort
	worker _ LDAPWorker newTo: aHostname port: aPort.
	worker connect.
	workerThread _ worker start.
! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 13:55'!
disconnect
	self unbind.
	worker disconnect.
	workerThread terminate! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: 'RHE 2/5/2004 22:37'!
bindAs: aUsername credentials: aPassword method: anAuthMethod
	^ LDAPBindRequest new initializeWith: worker username: aUsername credentials: aPassword method: anAuthMethod! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: 'RHE 2/5/2004 23:00'!
bindAs: aUsername credentials: aPassword
	^ self bindAs: aUsername credentials: aPassword method: nil
! !

!LDAPConnection class methodsFor: 'as yet unclassified' stamp: 'RHE 2/5/2004 21:47'!
to: aHostname
	^ self to: aHostname port: 389! !

!LDAPConnection methodsFor: 'as yet unclassified' stamp: 'RHE 2/12/2004 14:20'!
unbind
	LDAPUnbindRequest new initializeWith: worker! !

!LDAPConnection class methodsFor: 'as yet unclassified' stamp: 'RHE 2/5/2004 21:46'!
to: aHostname port: aPort
	| obj |
	obj _ self basicNew initializeWithHost: aHostname port: aPort.
	^ obj! !
